@@ -45,7 +45,7 @@ void onAboutClicked(HildonButton * button, AppData *data) {
 	strcat(infoLabel, "\n");
 	strcat(infoLabel, "\n");
 	strcat(infoLabel, "with contributions by\n");
-	strcat(infoLabel, "<i>hcm</i> (segfault patch)");
+	strcat(infoLabel, "<i>hcm</i> (various patches)");
 
 	hildon_banner_show_information_with_markup(
 			GTK_WIDGET(data->journeyWindow),
@@ -146,7 +146,7 @@ void onGPSLocation(LocationGPSDevice *device, AppData * data)
 	if (!device)
 		return;
 
-	if (device->fix) {
+	if (device->fix && device->status == LOCATION_GPS_DEVICE_STATUS_FIX) {
 
 		if (device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET) {
 
@@ -243,6 +243,18 @@ void onStartSearchClicked(HildonButton * button, AppData *data) {
 	}
 }
 
+/**
+ *
+ * OnClick Event for a HildonButton
+ * Handles swapping of the from/to stations
+ */
+void onSwapClicked(HildonButton * button, AppData *data) {
+	gchar *tmp = g_strdup_printf("%s", hildon_button_get_value(HILDON_BUTTON(data->toButton)));
+	hildon_button_set_value(HILDON_BUTTON(data->toButton),
+				hildon_button_get_value(HILDON_BUTTON(data->fromButton)));
+	hildon_button_set_value(HILDON_BUTTON(data->fromButton), tmp);
+}
+
 /**
  * onClick Event hanlder for GtkButton
  * Function to handle the selection of an entry of the results page
@@ -717,6 +729,15 @@ void initDepartureMonitorGui(AppData *data) {
 
 	/* Create HildonWindow and set it to HildonProgram */
 	data->departureWindow = HILDON_WINDOW(hildon_stackable_window_new());
+	hildon_program_add_window(data->program, data->departureWindow);
+
+	/* Quit program when window is closed. */
+	g_signal_connect (G_OBJECT (data->departureWindow), "delete_event",
+			G_CALLBACK (gtk_main_quit), NULL);
+
+	/* Quit program when window is otherwise destroyed. */
+	g_signal_connect (G_OBJECT (data->departureWindow), "destroy",
+			G_CALLBACK (gtk_main_quit), NULL);
 
 }
 /**
@@ -726,11 +747,24 @@ void initJourneyPlanerGui(AppData *data) {
 
 	/* Create HildonWindow and set it to HildonProgram */
 	data->journeyWindow = HILDON_WINDOW(hildon_stackable_window_new());
+	hildon_program_add_window(data->program, data->journeyWindow);
+
+	/* Quit program when window is closed. */
+	g_signal_connect (G_OBJECT (data->journeyWindow), "delete_event",
+			G_CALLBACK (gtk_main_quit), NULL);
+
+	/* Quit program when window is otherwise destroyed. */
+	g_signal_connect (G_OBJECT (data->journeyWindow), "destroy",
+			G_CALLBACK (gtk_main_quit), NULL);
 
 	/* create vertical container to stack the other containers vertically and add to main view */
 	data->verticalContainer = gtk_vbox_new(FALSE, 4);
 	gtk_container_add(GTK_CONTAINER(data->journeyWindow), data->verticalContainer);
 
+	/* create stations container */
+	data->stationsVContainer = gtk_hbox_new(FALSE, 4);
+	data->stationsHContainer = gtk_vbox_new(FALSE, 4);
+
 	/* create "from" elements */
 	data->fromContainer = gtk_hbox_new(FALSE, 4);
 
@@ -751,7 +785,7 @@ void initJourneyPlanerGui(AppData *data) {
 	gtk_box_pack_start(GTK_BOX (data->fromContainer), data->fromButton, TRUE,
 			TRUE, 0);
 
-	gtk_box_pack_start(GTK_BOX (data->verticalContainer), data->fromContainer,
+	gtk_box_pack_start(GTK_BOX (data->stationsHContainer), data->fromContainer,
 			TRUE, TRUE, 0);
 
 	/* create "to" elements */
@@ -774,9 +808,17 @@ void initJourneyPlanerGui(AppData *data) {
 	gtk_box_pack_start(GTK_BOX (data->toContainer), data->toButton, TRUE, TRUE,
 			0);
 
-	gtk_box_pack_start(GTK_BOX (data->verticalContainer), data->toContainer,
+	gtk_box_pack_start(GTK_BOX (data->stationsHContainer), data->toContainer,
 			TRUE, TRUE, 0);
 
+	gtk_box_pack_start(GTK_BOX(data->stationsVContainer), data->stationsHContainer, TRUE, TRUE, 0);
+
+	/* create swap button */
+	data->swapButton = gtk_button_new_with_label(_("\xe2\x86\x95"));
+	g_signal_connect(G_OBJECT(data->swapButton), "clicked", G_CALLBACK(onSwapClicked), data);
+	gtk_box_pack_start(GTK_BOX(data->stationsVContainer), data->swapButton, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(data->verticalContainer), data->stationsVContainer, TRUE, TRUE, 0);
+
 	/* create "time" elements */
 	data->timeContainer = gtk_hbox_new(FALSE, 4);
 
