@@ -42,7 +42,7 @@
 #include <curl/curl.h>
 
 #define APP_NAME "fahrplan"
-#define APP_VER "0.0.6"
+#define APP_VER "0.0.7"
 #define APP_SERVICE "com.nokia.fahrplan"
 #define APP_METHOD "/com/nokia/fahrplan"
 
@@ -104,6 +104,9 @@ struct _AppData {
 	HildonWindow *detailsWindow;
 
 	GtkWidget *verticalContainer;
+	GtkWidget *stationsVContainer;
+	GtkWidget *stationsHContainer;
+	GtkWidget *swapButton;
 	GtkWidget *fromContainer;
 	GtkWidget *fromSelector;
 	GtkWidget *fromButton;
